package positron;

import java.util.List;

import android.app.Activity;
import android.app.Application;
import android.app.ApplicationContext;
import android.app.Instrumentation.ActivityMonitor;
import android.app.Instrumentation.ActivityResult;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.IBinder;
import android.view.KeyEvent;
import android.view.View;

public abstract class TestCase extends junit.framework.TestCase {

	// key codes with shorter names.
	/** KeyEvent.KEYCODE_DPAD_CENTER */ public final static int CLICK = KeyEvent.KEYCODE_DPAD_CENTER;
	/** KeyEvent.KEYCODE_DPAD_CENTER */ public final static int CENTER = KeyEvent.KEYCODE_DPAD_CENTER;
	/** KeyEvent.KEYCODE_DPAD_UP */ public final static int UP = KeyEvent.KEYCODE_DPAD_UP;
	/** KeyEvent.KEYCODE_DPAD_DOWN */ public final static int DOWN = KeyEvent.KEYCODE_DPAD_DOWN;
	/** KeyEvent.KEYCODE_DPAD_LEFT */ public final static int LEFT = KeyEvent.KEYCODE_DPAD_LEFT;
	/** KeyEvent.KEYCODE_DPAD_RIGHT */ public final static int RIGHT = KeyEvent.KEYCODE_DPAD_RIGHT;

	/** KeyEvent.KEYCODE_BACK */ public final static int BACK = KeyEvent.KEYCODE_BACK;
	/** KeyEvent.KEYCODE_CALL */ public final static int CALL = KeyEvent.KEYCODE_CALL;
	/** KeyEvent.KEYCODE_CAMERA */ public final static int CAMERA = KeyEvent.KEYCODE_CAMERA;
	/** KeyEvent.KEYCODE_CAP */ public final static int CAP = KeyEvent.KEYCODE_CAP;
	/** KeyEvent.KEYCODE_CLEAR */ public final static int CLEAR = KeyEvent.KEYCODE_CLEAR;
	/** KeyEvent.KEYCODE_ENDCALL */ public final static int ENDCALL = KeyEvent.KEYCODE_ENDCALL;
	/** KeyEvent.KEYCODE_ENVELOPE */ public final static int ENVELOPE = KeyEvent.KEYCODE_ENVELOPE;
	/** KeyEvent.KEYCODE_EXPLORER */ public final static int EXPLORER = KeyEvent.KEYCODE_EXPLORER;
	/** KeyEvent.KEYCODE_ALT */ public final static int ALT = KeyEvent.KEYCODE_ALT;
	/** KeyEvent.KEYCODE_HOME */ public final static int HOME = KeyEvent.KEYCODE_HOME;
	/** KeyEvent.KEYCODE_NEWLINE */ public final static int NEWLINE = KeyEvent.KEYCODE_NEWLINE;
	/** KeyEvent.KEYCODE_NUM */ public final static int NUM = KeyEvent.KEYCODE_NUM;
	/** KeyEvent.KEYCODE_POWER */ public final static int POWER = KeyEvent.KEYCODE_POWER;
	/** KeyEvent.KEYCODE_SOFT_LEFT */ public final static int SOFT_LEFT = KeyEvent.KEYCODE_SOFT_LEFT;
	/** KeyEvent.KEYCODE_SOFT_RIGHT */ public final static int SOFT_RIGHT = KeyEvent.KEYCODE_SOFT_RIGHT;
	/** KeyEvent.KEYCODE_SYM */ public final static int SYM = KeyEvent.KEYCODE_SYM;
	/** KeyEvent.KEYCODE_TAB */ public final static int TAB = KeyEvent.KEYCODE_TAB;
	/** KeyEvent.KEYCODE_VOLUME_DOWN */ public final static int VOLUME_UP = KeyEvent.KEYCODE_VOLUME_DOWN;
	/** KeyEvent.KEYCODE_VOLUME_UP */ public final static int VOLUME_DOWN = KeyEvent.KEYCODE_VOLUME_UP;

	private Positron positron;

	// by DTM
	// for compatibility with generated TestCases
	public TestCase(String name) {
		super(name);
	}

	public TestCase() {
		super();
	}
	
	public void setInstrumentation(Positron positron) {
		this.positron = positron;
	}

	/** finishAll(); */
	@Override
	protected void tearDown() throws Exception {
		finishAll();
	}

	/**
	 * Get the most recent unfinished activity in the target package.
	 *
	 * This isn't quite correct.  The instrumentation can only detect new
	 * activities in the target package that list the package as a category.
	 *
	 * So, if the tested package is "my.android.app", then each activity declared
	 * in it's AndroidManifest.xml needs a <category android:value="my.android.app" />
	 * tag within it's intent-filter.
	 *
	 * Also remember, under all circumstances the instrumentation can only detect
	 * activities declared in the target package, according to the targetPackage
	 * attribute of the <instrumentation> tag positron was declared with.  Other
	 * started activities, such as contacts or the system browser, will have to
	 * be mocked out instead.  See the Instrumentation.ActivityMonitor documentation
	 * on how to do that.
	 */
	protected Activity activity() {
		return positron.activity();
	}

	/**
	 * Get all the unfinished activities in the target package, most recent first.
	 *
	 * The same restrictions described in activity() apply here.
	 *
	 * The list is unmodifiable.
	 */
	protected List<Activity> activities() {
		return positron.activities();
	}

	/**
	 * Finish the given activity by calling finish() on the main thread.
	 * This method will resume momentarily if needed.
	 */
	protected void finish(Activity activity) {
		positron.finish(activity);
	}

	/** finish(activity()), resuming momentarily as needed. */
	protected void finish() {
		positron.finish();
	}

	/**
	 * finish() all unfinished activities; useful in tearDown().
	 * This method will resume if needed but not immediately pause again.
	 */
	protected void finishAll() {
		positron.finishAll();
	}

	/**
	 * Pause execution of the target package.
	 *
	 * This is useful for inspecting the view state of the target package,
	 * since doing so while it is running can create race conditions. Resume
	 * execution with resume().
	 *
	 * Pausing an already-paused application is a safe no-op.
	 *
	 * Paused applications can't respond to events, nor execute synchronous
	 * commands such as sendStringSync (or any *Sync.)  Keep this in mind when
	 * writing tests.
	 *
	 * This version calls waitForIdleSync before pauing.  To skip waitForIdleSync,
	 * try pauseNow().
	 *
	 * Pausing is done by asynchronously calling Instrumentation.runOnMainSync on
	 * a blocking operation.
	 */
	protected void pause() {
		positron.pause();
	}

	/** Like pause, but do not implicitly call waitForIdleSync(). */
	protected void pauseNow() {
		positron.pauseNow();
	}

	/**
	 * Resume executing the target package if it has been paused.
	 *
	 * Resuming an unpaused package is a safe noop.
	 */
	protected void resume() {
		positron.resume();
	}

	/** Is the target package paused? */
	protected boolean paused() {
		return positron.paused();
	}

	/**
	 * Shortcut to startActivitySync the named class in the target context.
	 * @param className fully-qualified class name of the action to start.
	 */
	protected Activity startActivitySync(String className) {
        return positron.startActivitySync(className);
	}

	/** dupWindowBitmapSync, resuming momentarily if necessary. */
	protected Bitmap dupWindowBitmap(Activity activity) {
		return positron.dupWindowBitmap(activity);
	}

	/** invokeMenuActionSync, resuming momentarily if necessary. */
	protected void invokeMenuAction(Activity targetActivity, int id, int flag) {
		positron.invokeMenuAction(targetActivity, id, flag);
	}

	/** sendCharacterSync, resuming momentarily if necessary. */
	protected void sendCharacter(int keyCode) {
		positron.sendCharacter(keyCode);
	}

	/** sendKeySync, resuming momentarily if necessary. */
	protected void sendKey(KeyEvent event) {
		positron.sendKey(event);
	}

	/** sendKeyDownUpSync, resuming momentarily if necessary. */
	protected void sendKeyDownUp(int key) {
		positron.sendKeyDownUp(key);
	}

	/** sendKeyDownUp(KEYCODE_DPAD_CENTER) */
	protected void click() {
		positron.click();
	}

	/**
	 * sendKeyDownUpSync on all keys in order, resuming momentarily if necessary.
	 *
	 * Combined with the renamed constants available in positron.TestCase, this
	 * enables things like press(UP, UP, DOWN, DOWN, LEFT, RIGHT, LEFT, RIGHT, "ba")
	 * which may otherwise be cumbersome.
	 *
	 * @param keys A mixture of ints and Strings.  ints are sent with sendKeyDownUpSync,
	 * 		       Strings are send with sendStringSync.
	 * @throws IllegalArgumentException if something other than an int or String was passed.
	 */
	protected void press(Object... keys) {
		positron.press(keys);
	}

	protected <T> T at(Class<T> asA, Object from, String path) {
		return positron.at(asA, from, path);
	}

	protected <T> T at(Class<T> asA, String path) {
		return positron.at(asA, path);
	}

	protected Object at(String path) {
		return positron.at(path);
	}

	protected View viewAt(String path) {
		return positron.viewAt(path);
	}
	
	protected boolean booleanAt(String path) {
		return positron.booleanAt(path);
	}

	protected double doubleAt(String path) {
		return positron.doubleAt(path);
	}

	protected int intAt(String path) {
		return positron.intAt(path);
	}

	protected String stringAt(String path) {
		return positron.stringAt(path);
	}

	/** sendStringSync, resuming momentarily if necessary. */
	protected void sendString(String text) {
		positron.sendString(text);
	}

	/** startActivitySync, resuming momentarily if necessary. */
	protected Activity startActivity(Intent intent) {
		return positron.startActivity(intent);
	}

	/** waitForIdleSync, resuming momentarily if necessary. */
	protected void waitForIdle() {
		positron.waitForIdle();
	}

	protected void addMonitor(ActivityMonitor monitor) {
		positron.addMonitor(monitor);
	}

	protected ActivityMonitor addMonitor(IntentFilter filter, ActivityResult result, boolean block) {
		return positron.addMonitor(filter, result, block);
	}

	protected ActivityMonitor addMonitor(String cls, ActivityResult result, boolean block) {
		return positron.addMonitor(cls, result, block);
	}

	protected void callActivityOnCompleteThaw(Activity activity, Bundle icicle) {
		positron.callActivityOnCompleteThaw(activity, icicle);
	}

	protected void callActivityOnCreate(Activity activity, Bundle icicle) {
		positron.callActivityOnCreate(activity, icicle);
	}

	protected void callActivityOnFreeze(Activity activity, Bundle outState) {
		positron.callActivityOnFreeze(activity, outState);
	}

	protected void callActivityOnNewIntent(Activity activity, Intent intent) {
		positron.callActivityOnNewIntent(activity, intent);
	}

	protected void callActivityOnPause(Activity activity) {
		positron.callActivityOnPause(activity);
	}

	protected void callActivityOnPostCreate(Activity activity, Bundle icicle) {
		positron.callActivityOnPostCreate(activity, icicle);
	}

	protected void callActivityOnRestart(Activity activity) {
		positron.callActivityOnRestart(activity);
	}

	protected void callActivityOnResume(Activity activity) {
		positron.callActivityOnResume(activity);
	}

	protected void callActivityOnStart(Activity activity) {
		positron.callActivityOnStart(activity);
	}

	protected void callApplicationOnCreate(Application app) {
		positron.callApplicationOnCreate(app);
	}

	protected boolean checkMonitorHit(ActivityMonitor monitor, int minHits) {
		return positron.checkMonitorHit(monitor, minHits);
	}

	protected Bitmap dupWindowBitmapSync(Activity activity) {
		return positron.dupWindowBitmapSync(activity);
	}

	protected ActivityResult execStartActivity(ApplicationContext who,
			IBinder token, Activity target, Intent intent, int requestCode) {
		return positron.execStartActivity(who, token, target, intent,
				requestCode);
	}

	protected void finish(int resultCode, Bundle results) {
		positron.finish(resultCode, results);
	}

	protected void getAllocCounts(Bundle results) {
		positron.getAllocCounts(results);
	}

	protected Context getContext() {
		return positron.getContext();
	}

	protected Context getTargetContext() {
		return positron.getTargetContext();
	}

	protected void invokeMenuActionSync(Activity targetActivity, int id, int flag) {
		positron.invokeMenuActionSync(targetActivity, id, flag);
	}

	protected boolean isProfiling() {
		return positron.isProfiling();
	}

	protected Activity newActivity(ClassLoader cl, String className, Intent intent)
			throws InstantiationException, IllegalAccessException,
			ClassNotFoundException {
		return positron.newActivity(cl, className, intent);
	}

	protected Application newApplication(ClassLoader cl, String className)
			throws InstantiationException, IllegalAccessException,
			ClassNotFoundException {
		return positron.newApplication(cl, className);
	}

	protected void onCreate(Bundle arguments) {
		positron.onCreate(arguments);
	}

	protected void onDestroy() {
		positron.onDestroy();
	}

	protected boolean onException(Object obj, Throwable e) {
		return positron.onException(obj, e);
	}

	protected void onStart() {
		positron.onStart();
	}

	protected void removeMonitor(ActivityMonitor monitor) {
		positron.removeMonitor(monitor);
	}

	protected void runOnMainSync(Runnable runner) {
		positron.runOnMainSync(runner);
	}

	protected void sendCharacterSync(int keyCode) {
		positron.sendCharacterSync(keyCode);
	}

	protected void sendKeyDownUpSync(int key) {
		positron.sendKeyDownUpSync(key);
	}

	protected void sendKeySync(KeyEvent event) {
		positron.sendKeySync(event);
	}

	protected void sendStringSync(String text) {
		positron.sendStringSync(text);
	}

	protected void start() {
		positron.start();
	}

	protected Activity startActivitySync(Intent intent) {
		return positron.startActivitySync(intent);
	}

	protected void startAllocCounting() {
		positron.startAllocCounting();
	}

	protected void startProfiling() {
		positron.startProfiling();
	}

	protected void stopAllocCounting() {
		positron.stopAllocCounting();
	}

	protected void stopProfiling() {
		positron.stopProfiling();
	}

	protected void waitForIdle(Runnable recipient) {
		positron.waitForIdle(recipient);
	}

	protected void waitForIdleSync() {
		positron.waitForIdleSync();
	}

	protected Activity waitForMonitor(ActivityMonitor monitor) {
		return positron.waitForMonitor(monitor);
	}

	protected void backup() {
		positron.backup();
	}

	protected void backup(String database) {
		positron.backup(database);
	}

	protected void execSql(String database, int... scriptsAsRawResources) {
		positron.execSql(database, scriptsAsRawResources);
	}

	protected void restore() {
		positron.restore();
	}

	protected void restore(String database) {
		positron.restore(database);
	}
}
