package positron.harness;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Enumeration;

import junit.framework.TestFailure;
import junit.framework.TestResult;
import positron.Positron;
import positron.TestCase;
import android.os.Bundle;
import android.util.Log;

/**
 * TestResult that injects instrumentation.
 *
 * @author philhsmith
 */
public class InstrumentedTestResult extends TestResult {
	private static final String TAG = Positron.TAG;

	private final Positron instrumentation;

	public InstrumentedTestResult(Positron instrumentation) {
		this.instrumentation = instrumentation;
	}

	@Override
	protected void run(junit.framework.TestCase test) {
		if (test instanceof TestCase) {
			TestCase positronTest = (TestCase)test;
			positronTest.setInstrumentation(instrumentation);
		}

		Log.i(TAG, "-- " + test.getClass().getSimpleName() + "." + test.getName() + " --");

		super.run(test);
	}

	public Bundle toBundle() {
		Bundle bundle = new Bundle();

		bundle.putInt("runCount", runCount());

		StringWriter stringWriter = new StringWriter();
		StringBuffer buf = stringWriter.getBuffer();
		PrintWriter err = new PrintWriter(stringWriter);

		bundle.putParcelable("failures", toBundle(failures(), err, buf));
		bundle.putParcelable("errors", toBundle(errors(), err, buf));

		return bundle;
	}

	@SuppressWarnings("unchecked")
	private Bundle toBundle(Enumeration failures, PrintWriter err, StringBuffer buf) {
		Bundle results = new Bundle();

		while (failures.hasMoreElements()) {
			TestFailure failure = (TestFailure)failures.nextElement();

			failure.thrownException().printStackTrace(err);
			err.flush();

			results.putString(failure.failedTest().toString(), buf.toString());

			buf.replace(0, buf.length(), "");
		}

		return results;
	}
}